import { Asset } from '../asset/asset.js';
import { Http, http } from '../../platform/net/http.js';
import { GSplatResource } from './gsplat-resource.js';
import { GSplatSogsData } from '../../scene/gsplat/gsplat-sogs-data.js';

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
/**
 * @import { AppBase } from '../app-base.js'
 * @import { ResourceHandlerCallback } from '../handlers/handler.js'
 */ class SogsParser {
    loadTextures(url, callback, asset, meta) {
        var _this = this;
        return _async_to_generator(function*() {
            var _textures_shN__resource, _textures_shN_, _textures_shN, _textures_shN_1, _textures_shN1, _textures_shN_2, _textures_shN2, _textures_shN_3, _textures_shN3, _asset_data;
            var { assets } = _this.app;
            var subs = [
                'means',
                'opacities',
                'quats',
                'scales',
                'sh0',
                'shN'
            ];
            var textures = {};
            var promises = [];
            subs.forEach((sub)=>{
                var _meta_sub;
                var _meta_sub_files;
                var files = (_meta_sub_files = (_meta_sub = meta[sub]) == null ? void 0 : _meta_sub.files) != null ? _meta_sub_files : [];
                textures[sub] = files.map((filename)=>{
                    var _asset_options_mapUrl, _asset_options;
                    var _asset_options_mapUrl1;
                    var texture = new Asset(filename, 'texture', {
                        url: (_asset_options_mapUrl1 = (_asset_options = asset.options) == null ? void 0 : (_asset_options_mapUrl = _asset_options.mapUrl) == null ? void 0 : _asset_options_mapUrl.call(_asset_options, filename)) != null ? _asset_options_mapUrl1 : new URL(filename, url.load).toString(),
                        filename
                    }, {
                        mipmaps: false
                    });
                    var promise = new Promise((resolve, reject)=>{
                        texture.on('load', ()=>resolve(null));
                        texture.on('error', (err)=>reject(err));
                    });
                    assets.add(texture);
                    assets.load(texture);
                    promises.push(promise);
                    return texture;
                });
            });
            // wait for all textures to complete loading
            yield Promise.allSettled(promises);
            // sh palette has 64 sh entries per row. use width to calculate number of bands
            var widths = {
                192: 1,
                512: 2,
                960: 3 // 64 * 15
            };
            // construct the gsplat resource
            var data = new GSplatSogsData();
            data.meta = meta;
            data.numSplats = meta.means.shape[0];
            var _widths_textures_shN__resource_width;
            data.shBands = (_widths_textures_shN__resource_width = widths[(_textures_shN = textures.shN) == null ? void 0 : (_textures_shN_ = _textures_shN[0]) == null ? void 0 : (_textures_shN__resource = _textures_shN_.resource) == null ? void 0 : _textures_shN__resource.width]) != null ? _widths_textures_shN__resource_width : 0;
            data.means_l = textures.means[0].resource;
            data.means_u = textures.means[1].resource;
            data.opacities = textures.opacities[0].resource;
            data.quats = textures.quats[0].resource;
            data.scales = textures.scales[0].resource;
            data.sh0 = textures.sh0[0].resource;
            data.sh_centroids = (_textures_shN1 = textures.shN) == null ? void 0 : (_textures_shN_1 = _textures_shN1[0]) == null ? void 0 : _textures_shN_1.resource;
            data.sh_labels_l = (_textures_shN2 = textures.shN) == null ? void 0 : (_textures_shN_2 = _textures_shN2[1]) == null ? void 0 : _textures_shN_2.resource;
            data.sh_labels_u = (_textures_shN3 = textures.shN) == null ? void 0 : (_textures_shN_3 = _textures_shN3[2]) == null ? void 0 : _textures_shN_3.resource;
            var resource = new GSplatResource(_this.app, ((_asset_data = asset.data) == null ? void 0 : _asset_data.decompress) ? data.decompress() : data, []);
            callback(null, resource);
        })();
    }
    /**
     * @param {object} url - The URL of the resource to load.
     * @param {string} url.load - The URL to use for loading the resource.
     * @param {string} url.original - The original URL useful for identifying the resource type.
     * @param {ResourceHandlerCallback} callback - The callback used when
     * the resource is loaded or an error occurs.
     * @param {Asset} asset - Container asset.
     */ load(url, callback, asset) {
        var _asset_data;
        if ((_asset_data = asset.data) == null ? void 0 : _asset_data.means) {
            // user can specify meta.json in asset data
            this.loadTextures(url, callback, asset, asset.data);
        } else {
            // otherwise download meta.json using asset url
            if (typeof url === 'string') {
                url = {
                    load: url,
                    original: url
                };
            }
            // we need to specify JSON for blob URLs
            var options = {
                retry: this.maxRetries > 0,
                maxRetries: this.maxRetries,
                responseType: Http.ResponseType.JSON
            };
            http.get(url.load, options, (err, meta)=>{
                if (!err) {
                    this.loadTextures(url, callback, asset, meta);
                } else {
                    callback("Error loading gsplat meta: " + url.original + " [" + err + "]");
                }
            });
        }
    }
    /**
     * @param {AppBase} app - The app instance.
     * @param {number} maxRetries - Maximum amount of retries.
     */ constructor(app, maxRetries){
        this.app = app;
        this.maxRetries = maxRetries;
    }
}

export { SogsParser };
