import { SceneParser } from './parsers/scene.js';

/**
 * @import { AppBase } from './app-base.js'
 * @import { Entity } from './entity.js'
 */ /**
 * Create a Template resource from raw database data.
 */ class Template {
    /**
     * Create an instance of this template.
     *
     * @returns {Entity} The root entity of the created instance.
     */ instantiate() {
        if (!this._templateRoot) {
            this._parseTemplate();
        }
        return this._templateRoot.clone();
    }
    /** @private */ _parseTemplate() {
        var parser = new SceneParser(this._app, true);
        this._templateRoot = parser.parse(this._data);
    }
    /**
     * Create a new Template instance.
     *
     * @param {AppBase} app - The application.
     * @param {object} data - Asset data from the database.
     */ constructor(app, data){
        /**
     * @type {Entity|null}
     * @private
     */ this._templateRoot = null;
        this._app = app;
        this._data = data;
    }
}

export { Template };
