import { platform } from '../../core/platform.js';
import { Vec3 } from '../../core/math/vec3.js';
import { Vec4 } from '../../core/math/vec4.js';
import { Ray } from '../../core/shape/ray.js';
import { Mouse } from '../../platform/input/mouse.js';
import { getTouchTargetCoords } from '../../platform/input/touch-event.js';
import { getApplication } from '../globals.js';

var targetX, targetY;
var vecA = new Vec3();
var vecB = new Vec3();
var rayA = new Ray();
var rayB = new Ray();
var rayC = new Ray();
rayA.end = new Vec3();
rayB.end = new Vec3();
rayC.end = new Vec3();
var _pq = new Vec3();
var _pa = new Vec3();
var _pb = new Vec3();
var _pc = new Vec3();
var _pd = new Vec3();
var _m = new Vec3();
var _au = new Vec3();
var _bv = new Vec3();
var _cw = new Vec3();
var _ir = new Vec3();
var _sct = new Vec3();
var _accumulatedScale = new Vec3();
var _paddingTop = new Vec3();
var _paddingBottom = new Vec3();
var _paddingLeft = new Vec3();
var _paddingRight = new Vec3();
var _cornerBottomLeft = new Vec3();
var _cornerBottomRight = new Vec3();
var _cornerTopRight = new Vec3();
var _cornerTopLeft = new Vec3();
var ZERO_VEC4 = new Vec4();
function scalarTriple(p1, p2, p3) {
		return _sct.cross(p1, p2).dot(p3);
}
function intersectLineQuad(p, q, corners) {
		_pq.sub2(q, p);
		_pa.sub2(corners[0], p);
		_pb.sub2(corners[1], p);
		_pc.sub2(corners[2], p);
		_m.cross(_pc, _pq);
		var v = _pa.dot(_m);
		var u;
		var w;
		if (v >= 0) {
				u = -_pb.dot(_m);
				if (u < 0) {
						return -1;
				}
				w = scalarTriple(_pq, _pb, _pa);
				if (w < 0) {
						return -1;
				}
				var denom = 1.0 / (u + v + w);
				_au.copy(corners[0]).mulScalar(u * denom);
				_bv.copy(corners[1]).mulScalar(v * denom);
				_cw.copy(corners[2]).mulScalar(w * denom);
				_ir.copy(_au).add(_bv).add(_cw);
		} else {
				_pd.sub2(corners[3], p);
				u = _pd.dot(_m);
				if (u < 0) {
						return -1;
				}
				w = scalarTriple(_pq, _pa, _pd);
				if (w < 0) {
						return -1;
				}
				v = -v;
				var denom1 = 1.0 / (u + v + w);
				_au.copy(corners[0]).mulScalar(u * denom1);
				_bv.copy(corners[3]).mulScalar(v * denom1);
				_cw.copy(corners[2]).mulScalar(w * denom1);
				_ir.copy(_au).add(_bv).add(_cw);
		}
		if (_pq.sub2(corners[0], corners[2]).lengthSq() < 0.0001 * 0.0001) return -1;
		if (_pq.sub2(corners[1], corners[3]).lengthSq() < 0.0001 * 0.0001) return -1;
		return _ir.sub(p).lengthSq();
}
class ElementInputEvent {
		stopPropagation() {
				this._stopPropagation = true;
				if (this.event) {
						this.event.stopImmediatePropagation();
						this.event.stopPropagation();
				}
		}
		constructor(event, element, camera){
				this.event = event;
				this.element = element;
				this.camera = camera;
				this._stopPropagation = false;
		}
}
class ElementMouseEvent extends ElementInputEvent {
		constructor(event, element, camera, x, y, lastX, lastY){
				super(event, element, camera);
				this.x = x;
				this.y = y;
				this.ctrlKey = event.ctrlKey || false;
				this.altKey = event.altKey || false;
				this.shiftKey = event.shiftKey || false;
				this.metaKey = event.metaKey || false;
				this.button = event.button;
				if (Mouse.isPointerLocked()) {
						this.dx = event.movementX || event.webkitMovementX || event.mozMovementX || 0;
						this.dy = event.movementY || event.webkitMovementY || event.mozMovementY || 0;
				} else {
						this.dx = x - lastX;
						this.dy = y - lastY;
				}
				this.wheelDelta = 0;
				if (event.type === 'wheel') {
						if (event.deltaY > 0) {
								this.wheelDelta = 1;
						} else if (event.deltaY < 0) {
								this.wheelDelta = -1;
						}
				}
		}
}
class ElementTouchEvent extends ElementInputEvent {
		constructor(event, element, camera, x, y, touch){
				super(event, element, camera);
				this.touches = event.touches;
				this.changedTouches = event.changedTouches;
				this.x = x;
				this.y = y;
				this.touch = touch;
		}
}
class ElementSelectEvent extends ElementInputEvent {
		constructor(event, element, camera, inputSource){
				super(event, element, camera);
				this.inputSource = inputSource;
		}
}
class ElementInput {
		set enabled(value) {
				this._enabled = value;
		}
		get enabled() {
				return this._enabled;
		}
		set app(value) {
				this._app = value;
		}
		get app() {
				return this._app || getApplication();
		}
		attach(domElement) {
				if (this._attached) {
						this._attached = false;
						this.detach();
				}
				this._target = domElement;
				this._attached = true;
				var opts = platform.passiveEvents ? {
						passive: true
				} : false;
				if (this._useMouse) {
						window.addEventListener('mouseup', this._upHandler, opts);
						window.addEventListener('mousedown', this._downHandler, opts);
						window.addEventListener('mousemove', this._moveHandler, opts);
						window.addEventListener('wheel', this._wheelHandler, opts);
				}
				if (this._useTouch && platform.touch) {
						this._target.addEventListener('touchstart', this._touchstartHandler, opts);
						this._target.addEventListener('touchend', this._touchendHandler, false);
						this._target.addEventListener('touchmove', this._touchmoveHandler, false);
						this._target.addEventListener('touchcancel', this._touchcancelHandler, false);
				}
				this.attachSelectEvents();
		}
		attachSelectEvents() {
				if (!this._selectEventsAttached && this._useXr && this.app && this.app.xr && this.app.xr.supported) {
						if (!this._clickedEntities) {
								this._clickedEntities = {};
						}
						this._selectEventsAttached = true;
						this.app.xr.on('start', this._onXrStart, this);
				}
		}
		detach() {
				if (!this._attached) return;
				this._attached = false;
				var opts = platform.passiveEvents ? {
						passive: true
				} : false;
				if (this._useMouse) {
						window.removeEventListener('mouseup', this._upHandler, opts);
						window.removeEventListener('mousedown', this._downHandler, opts);
						window.removeEventListener('mousemove', this._moveHandler, opts);
						window.removeEventListener('wheel', this._wheelHandler, opts);
				}
				if (this._useTouch) {
						this._target.removeEventListener('touchstart', this._touchstartHandler, opts);
						this._target.removeEventListener('touchend', this._touchendHandler, false);
						this._target.removeEventListener('touchmove', this._touchmoveHandler, false);
						this._target.removeEventListener('touchcancel', this._touchcancelHandler, false);
				}
				if (this._selectEventsAttached) {
						this._selectEventsAttached = false;
						this.app.xr.off('start', this._onXrStart, this);
						this.app.xr.off('end', this._onXrEnd, this);
						this.app.xr.off('update', this._onXrUpdate, this);
						this.app.xr.input.off('selectstart', this._onSelectStart, this);
						this.app.xr.input.off('selectend', this._onSelectEnd, this);
						this.app.xr.input.off('remove', this._onXrInputRemove, this);
				}
				this._target = null;
		}
		addElement(element) {
				if (this._elements.indexOf(element) === -1) {
						this._elements.push(element);
				}
		}
		removeElement(element) {
				var idx = this._elements.indexOf(element);
				if (idx !== -1) {
						this._elements.splice(idx, 1);
				}
		}
		_handleUp(event) {
				if (!this._enabled) return;
				if (Mouse.isPointerLocked()) {
						return;
				}
				this._calcMouseCoords(event);
				this._onElementMouseEvent('mouseup', event);
		}
		_handleDown(event) {
				if (!this._enabled) return;
				if (Mouse.isPointerLocked()) {
						return;
				}
				this._calcMouseCoords(event);
				this._onElementMouseEvent('mousedown', event);
		}
		_handleMove(event) {
				if (!this._enabled) return;
				this._calcMouseCoords(event);
				this._onElementMouseEvent('mousemove', event);
				this._lastX = targetX;
				this._lastY = targetY;
		}
		_handleWheel(event) {
				if (!this._enabled) return;
				this._calcMouseCoords(event);
				this._onElementMouseEvent('mousewheel', event);
		}
		_determineTouchedElements(event) {
				var touchedElements = {};
				var cameras = this.app.systems.camera.cameras;
				for(var i = cameras.length - 1; i >= 0; i--){
						var camera = cameras[i];
						var done = 0;
						var len = event.changedTouches.length;
						for(var j = 0; j < len; j++){
								if (touchedElements[event.changedTouches[j].identifier]) {
										done++;
										continue;
								}
								var coords = getTouchTargetCoords(event.changedTouches[j]);
								var element = this._getTargetElementByCoords(camera, coords.x, coords.y);
								if (element) {
										done++;
										touchedElements[event.changedTouches[j].identifier] = {
												element: element,
												camera: camera,
												x: coords.x,
												y: coords.y
										};
								}
						}
						if (done === len) {
								break;
						}
				}
				return touchedElements;
		}
		_handleTouchStart(event) {
				if (!this._enabled) return;
				var newTouchedElements = this._determineTouchedElements(event);
				for(var i = 0, len = event.changedTouches.length; i < len; i++){
						var touch = event.changedTouches[i];
						var newTouchInfo = newTouchedElements[touch.identifier];
						var oldTouchInfo = this._touchedElements[touch.identifier];
						if (newTouchInfo && (!oldTouchInfo || newTouchInfo.element !== oldTouchInfo.element)) {
								this._fireEvent(event.type, new ElementTouchEvent(event, newTouchInfo.element, newTouchInfo.camera, newTouchInfo.x, newTouchInfo.y, touch));
								this._touchesForWhichTouchLeaveHasFired[touch.identifier] = false;
						}
				}
				for(var touchId in newTouchedElements){
						this._touchedElements[touchId] = newTouchedElements[touchId];
				}
		}
		_handleTouchEnd(event) {
				if (!this._enabled) return;
				var cameras = this.app.systems.camera.cameras;
				for(var key in this._clickedEntities){
						delete this._clickedEntities[key];
				}
				for(var i = 0, len = event.changedTouches.length; i < len; i++){
						var touch = event.changedTouches[i];
						var touchInfo = this._touchedElements[touch.identifier];
						if (!touchInfo) {
								continue;
						}
						var element = touchInfo.element;
						var camera = touchInfo.camera;
						var x = touchInfo.x;
						var y = touchInfo.y;
						delete this._touchedElements[touch.identifier];
						delete this._touchesForWhichTouchLeaveHasFired[touch.identifier];
						var coords = getTouchTargetCoords(touch);
						for(var c = cameras.length - 1; c >= 0; c--){
								var hovered = this._getTargetElementByCoords(cameras[c], coords.x, coords.y);
								if (hovered === element) {
										if (!this._clickedEntities[element.entity.getGuid()]) {
												this._fireEvent('click', new ElementTouchEvent(event, element, camera, x, y, touch));
												this._clickedEntities[element.entity.getGuid()] = Date.now();
										}
								}
						}
						this._fireEvent(event.type, new ElementTouchEvent(event, element, camera, x, y, touch));
				}
		}
		_handleTouchMove(event) {
				event.preventDefault();
				if (!this._enabled) return;
				var newTouchedElements = this._determineTouchedElements(event);
				for(var i = 0, len = event.changedTouches.length; i < len; i++){
						var touch = event.changedTouches[i];
						var newTouchInfo = newTouchedElements[touch.identifier];
						var oldTouchInfo = this._touchedElements[touch.identifier];
						if (oldTouchInfo) {
								var coords = getTouchTargetCoords(touch);
								if ((!newTouchInfo || newTouchInfo.element !== oldTouchInfo.element) && !this._touchesForWhichTouchLeaveHasFired[touch.identifier]) {
										this._fireEvent('touchleave', new ElementTouchEvent(event, oldTouchInfo.element, oldTouchInfo.camera, coords.x, coords.y, touch));
										this._touchesForWhichTouchLeaveHasFired[touch.identifier] = true;
								}
								this._fireEvent('touchmove', new ElementTouchEvent(event, oldTouchInfo.element, oldTouchInfo.camera, coords.x, coords.y, touch));
						}
				}
		}
		_onElementMouseEvent(eventType, event) {
				var element = null;
				var lastHovered = this._hoveredElement;
				this._hoveredElement = null;
				var cameras = this.app.systems.camera.cameras;
				var camera;
				for(var i = cameras.length - 1; i >= 0; i--){
						camera = cameras[i];
						element = this._getTargetElementByCoords(camera, targetX, targetY);
						if (element) {
								break;
						}
				}
				this._hoveredElement = element;
				if ((eventType === 'mousemove' || eventType === 'mouseup') && this._pressedElement) {
						this._fireEvent(eventType, new ElementMouseEvent(event, this._pressedElement, camera, targetX, targetY, this._lastX, this._lastY));
				} else if (element) {
						this._fireEvent(eventType, new ElementMouseEvent(event, element, camera, targetX, targetY, this._lastX, this._lastY));
						if (eventType === 'mousedown') {
								this._pressedElement = element;
						}
				}
				if (lastHovered !== this._hoveredElement) {
						if (lastHovered) {
								this._fireEvent('mouseleave', new ElementMouseEvent(event, lastHovered, camera, targetX, targetY, this._lastX, this._lastY));
						}
						if (this._hoveredElement) {
								this._fireEvent('mouseenter', new ElementMouseEvent(event, this._hoveredElement, camera, targetX, targetY, this._lastX, this._lastY));
						}
				}
				if (eventType === 'mouseup' && this._pressedElement) {
						if (this._pressedElement === this._hoveredElement) {
								var guid = this._hoveredElement.entity.getGuid();
								var fireClick = !this._clickedEntities;
								if (this._clickedEntities) {
										var lastTouchUp = this._clickedEntities[guid] || 0;
										var dt = Date.now() - lastTouchUp;
										fireClick = dt > 300;
										delete this._clickedEntities[guid];
								}
								if (fireClick) {
										this._fireEvent('click', new ElementMouseEvent(event, this._hoveredElement, camera, targetX, targetY, this._lastX, this._lastY));
								}
						}
						this._pressedElement = null;
				}
		}
		_onXrStart() {
				this.app.xr.on('end', this._onXrEnd, this);
				this.app.xr.on('update', this._onXrUpdate, this);
				this.app.xr.input.on('selectstart', this._onSelectStart, this);
				this.app.xr.input.on('selectend', this._onSelectEnd, this);
				this.app.xr.input.on('remove', this._onXrInputRemove, this);
		}
		_onXrEnd() {
				this.app.xr.off('update', this._onXrUpdate, this);
				this.app.xr.input.off('selectstart', this._onSelectStart, this);
				this.app.xr.input.off('selectend', this._onSelectEnd, this);
				this.app.xr.input.off('remove', this._onXrInputRemove, this);
		}
		_onXrUpdate() {
				if (!this._enabled) return;
				var inputSources = this.app.xr.input.inputSources;
				for(var i = 0; i < inputSources.length; i++){
						this._onElementSelectEvent('selectmove', inputSources[i], null);
				}
		}
		_onXrInputRemove(inputSource) {
				var hovered = this._selectedElements[inputSource.id];
				if (hovered) {
						inputSource._elementEntity = null;
						this._fireEvent('selectleave', new ElementSelectEvent(null, hovered, null, inputSource));
				}
				delete this._selectedElements[inputSource.id];
				delete this._selectedPressedElements[inputSource.id];
		}
		_onSelectStart(inputSource, event) {
				if (!this._enabled) return;
				this._onElementSelectEvent('selectstart', inputSource, event);
		}
		_onSelectEnd(inputSource, event) {
				if (!this._enabled) return;
				this._onElementSelectEvent('selectend', inputSource, event);
		}
		_onElementSelectEvent(eventType, inputSource, event) {
				var element;
				var hoveredBefore = this._selectedElements[inputSource.id];
				var hoveredNow;
				var cameras = this.app.systems.camera.cameras;
				var camera;
				if (inputSource.elementInput) {
						rayC.set(inputSource.getOrigin(), inputSource.getDirection());
						for(var i = cameras.length - 1; i >= 0; i--){
								camera = cameras[i];
								element = this._getTargetElementByRay(rayC, camera);
								if (element) {
										break;
								}
						}
				}
				inputSource._elementEntity = element || null;
				if (element) {
						this._selectedElements[inputSource.id] = element;
						hoveredNow = element;
				} else {
						delete this._selectedElements[inputSource.id];
				}
				if (hoveredBefore !== hoveredNow) {
						if (hoveredBefore) this._fireEvent('selectleave', new ElementSelectEvent(event, hoveredBefore, camera, inputSource));
						if (hoveredNow) this._fireEvent('selectenter', new ElementSelectEvent(event, hoveredNow, camera, inputSource));
				}
				var pressed = this._selectedPressedElements[inputSource.id];
				if (eventType === 'selectmove' && pressed) {
						this._fireEvent('selectmove', new ElementSelectEvent(event, pressed, camera, inputSource));
				}
				if (eventType === 'selectstart') {
						this._selectedPressedElements[inputSource.id] = hoveredNow;
						if (hoveredNow) this._fireEvent('selectstart', new ElementSelectEvent(event, hoveredNow, camera, inputSource));
				}
				if (!inputSource.elementInput && pressed) {
						delete this._selectedPressedElements[inputSource.id];
						if (hoveredBefore) {
								this._fireEvent('selectend', new ElementSelectEvent(event, pressed, camera, inputSource));
						}
				}
				if (eventType === 'selectend' && inputSource.elementInput) {
						delete this._selectedPressedElements[inputSource.id];
						if (pressed) {
								this._fireEvent('selectend', new ElementSelectEvent(event, pressed, camera, inputSource));
						}
						if (pressed && pressed === hoveredBefore) {
								this._fireEvent('click', new ElementSelectEvent(event, pressed, camera, inputSource));
						}
				}
		}
		_fireEvent(name, evt) {
				var element = evt.element;
				while(true){
						element.fire(name, evt);
						if (evt._stopPropagation) {
								break;
						}
						if (!element.entity.parent) {
								break;
						}
						element = element.entity.parent.element;
						if (!element) {
								break;
						}
				}
		}
		_calcMouseCoords(event) {
				var rect = this._target.getBoundingClientRect();
				var left = Math.floor(rect.left);
				var top = Math.floor(rect.top);
				targetX = event.clientX - left;
				targetY = event.clientY - top;
		}
		_sortElements(a, b) {
				var layerOrder = this.app.scene.layers.sortTransparentLayers(a.layers, b.layers);
				if (layerOrder !== 0) return layerOrder;
				if (a.screen && !b.screen) {
						return -1;
				}
				if (!a.screen && b.screen) {
						return 1;
				}
				if (!a.screen && !b.screen) {
						return 0;
				}
				if (a.screen.screen.screenSpace && !b.screen.screen.screenSpace) {
						return -1;
				}
				if (b.screen.screen.screenSpace && !a.screen.screen.screenSpace) {
						return 1;
				}
				return b.drawOrder - a.drawOrder;
		}
		_getTargetElementByCoords(camera, x, y) {
				var rayScreen = this._calculateRayScreen(x, y, camera, rayA) ? rayA : null;
				var ray3d = this._calculateRay3d(x, y, camera, rayB) ? rayB : null;
				return this._getTargetElement(camera, rayScreen, ray3d);
		}
		_getTargetElementByRay(ray, camera) {
				rayA.origin.copy(ray.origin);
				rayA.direction.copy(ray.direction);
				rayA.end.copy(rayA.direction).mulScalar(camera.farClip * 2).add(rayA.origin);
				var ray3d = rayA;
				var screenPos = camera.worldToScreen(ray3d.origin, vecA);
				var rayScreen = this._calculateRayScreen(screenPos.x, screenPos.y, camera, rayB) ? rayB : null;
				return this._getTargetElement(camera, rayScreen, ray3d);
		}
		_getTargetElement(camera, rayScreen, ray3d) {
				var result = null;
				var closestDistance3d = Infinity;
				this._elements.sort(this._sortHandler);
				for(var i = 0, len = this._elements.length; i < len; i++){
						var element = this._elements[i];
						if (!element.layers.some((v)=>camera.layersSet.has(v))) {
								continue;
						}
						if (element.screen && element.screen.screen.screenSpace) {
								if (!rayScreen) {
										continue;
								}
								var currentDistance = this._checkElement(rayScreen, element, true);
								if (currentDistance >= 0) {
										result = element;
										break;
								}
						} else {
								if (!ray3d) {
										continue;
								}
								var currentDistance1 = this._checkElement(ray3d, element, false);
								if (currentDistance1 >= 0) {
										if (currentDistance1 < closestDistance3d) {
												result = element;
												closestDistance3d = currentDistance1;
										}
										if (element.screen) {
												result = element;
												break;
										}
								}
						}
				}
				return result;
		}
		_calculateRayScreen(x, y, camera, ray) {
				var sw = this.app.graphicsDevice.width;
				var sh = this.app.graphicsDevice.height;
				var cameraWidth = camera.rect.z * sw;
				var cameraHeight = camera.rect.w * sh;
				var cameraLeft = camera.rect.x * sw;
				var cameraRight = cameraLeft + cameraWidth;
				var cameraBottom = (1 - camera.rect.y) * sh;
				var cameraTop = cameraBottom - cameraHeight;
				var _x = x * sw / this._target.clientWidth;
				var _y = y * sh / this._target.clientHeight;
				if (_x >= cameraLeft && _x <= cameraRight && _y <= cameraBottom && _y >= cameraTop) {
						_x = sw * (_x - cameraLeft) / cameraWidth;
						_y = sh * (_y - cameraTop) / cameraHeight;
						_y = sh - _y;
						ray.origin.set(_x, _y, 1);
						ray.direction.set(0, 0, -1);
						ray.end.copy(ray.direction).mulScalar(2).add(ray.origin);
						return true;
				}
				return false;
		}
		_calculateRay3d(x, y, camera, ray) {
				var sw = this._target.clientWidth;
				var sh = this._target.clientHeight;
				var cameraWidth = camera.rect.z * sw;
				var cameraHeight = camera.rect.w * sh;
				var cameraLeft = camera.rect.x * sw;
				var cameraRight = cameraLeft + cameraWidth;
				var cameraBottom = (1 - camera.rect.y) * sh;
				var cameraTop = cameraBottom - cameraHeight;
				var _x = x;
				var _y = y;
				if (x >= cameraLeft && x <= cameraRight && y <= cameraBottom && _y >= cameraTop) {
						_x = sw * (_x - cameraLeft) / cameraWidth;
						_y = sh * (_y - cameraTop) / cameraHeight;
						camera.screenToWorld(_x, _y, camera.nearClip, vecA);
						camera.screenToWorld(_x, _y, camera.farClip, vecB);
						ray.origin.copy(vecA);
						ray.direction.set(0, 0, -1);
						ray.end.copy(vecB);
						return true;
				}
				return false;
		}
		_checkElement(ray, element, screen) {
				if (element.maskedBy) {
						if (this._checkElement(ray, element.maskedBy.element, screen) < 0) {
								return -1;
						}
				}
				var scale;
				if (screen) {
						scale = ElementInput.calculateScaleToScreen(element);
				} else {
						scale = ElementInput.calculateScaleToWorld(element);
				}
				var corners = ElementInput.buildHitCorners(element, screen ? element.screenCorners : element.worldCorners, scale);
				return intersectLineQuad(ray.origin, ray.end, corners);
		}
		static buildHitCorners(element, screenOrWorldCorners, scale) {
				var hitCorners = screenOrWorldCorners;
				var button = element.entity && element.entity.button;
				if (button) {
						var hitPadding = element.entity.button.hitPadding || ZERO_VEC4;
						_paddingTop.copy(element.entity.up);
						_paddingBottom.copy(_paddingTop).mulScalar(-1);
						_paddingRight.copy(element.entity.right);
						_paddingLeft.copy(_paddingRight).mulScalar(-1);
						_paddingTop.mulScalar(hitPadding.w * scale.y);
						_paddingBottom.mulScalar(hitPadding.y * scale.y);
						_paddingRight.mulScalar(hitPadding.z * scale.x);
						_paddingLeft.mulScalar(hitPadding.x * scale.x);
						_cornerBottomLeft.copy(hitCorners[0]).add(_paddingBottom).add(_paddingLeft);
						_cornerBottomRight.copy(hitCorners[1]).add(_paddingBottom).add(_paddingRight);
						_cornerTopRight.copy(hitCorners[2]).add(_paddingTop).add(_paddingRight);
						_cornerTopLeft.copy(hitCorners[3]).add(_paddingTop).add(_paddingLeft);
						hitCorners = [
								_cornerBottomLeft,
								_cornerBottomRight,
								_cornerTopRight,
								_cornerTopLeft
						];
				}
				if (scale.x < 0) {
						var left = hitCorners[2].x;
						var right = hitCorners[0].x;
						hitCorners[0].x = left;
						hitCorners[1].x = right;
						hitCorners[2].x = right;
						hitCorners[3].x = left;
				}
				if (scale.y < 0) {
						var bottom = hitCorners[2].y;
						var top = hitCorners[0].y;
						hitCorners[0].y = bottom;
						hitCorners[1].y = bottom;
						hitCorners[2].y = top;
						hitCorners[3].y = top;
				}
				if (scale.z < 0) {
						var x = hitCorners[2].x;
						var y = hitCorners[2].y;
						var z = hitCorners[2].z;
						hitCorners[2].x = hitCorners[0].x;
						hitCorners[2].y = hitCorners[0].y;
						hitCorners[2].z = hitCorners[0].z;
						hitCorners[0].x = x;
						hitCorners[0].y = y;
						hitCorners[0].z = z;
				}
				return hitCorners;
		}
		static calculateScaleToScreen(element) {
				var current = element.entity;
				var screenScale = element.screen.screen.scale;
				_accumulatedScale.set(screenScale, screenScale, screenScale);
				while(current && !current.screen){
						_accumulatedScale.mul(current.getLocalScale());
						current = current.parent;
				}
				return _accumulatedScale;
		}
		static calculateScaleToWorld(element) {
				var current = element.entity;
				_accumulatedScale.set(1, 1, 1);
				while(current){
						_accumulatedScale.mul(current.getLocalScale());
						current = current.parent;
				}
				return _accumulatedScale;
		}
		constructor(domElement, options){
				this._app = null;
				this._attached = false;
				this._target = null;
				this._enabled = true;
				this._lastX = 0;
				this._lastY = 0;
				this._upHandler = this._handleUp.bind(this);
				this._downHandler = this._handleDown.bind(this);
				this._moveHandler = this._handleMove.bind(this);
				this._wheelHandler = this._handleWheel.bind(this);
				this._touchstartHandler = this._handleTouchStart.bind(this);
				this._touchendHandler = this._handleTouchEnd.bind(this);
				this._touchcancelHandler = this._touchendHandler;
				this._touchmoveHandler = this._handleTouchMove.bind(this);
				this._sortHandler = this._sortElements.bind(this);
				this._elements = [];
				this._hoveredElement = null;
				this._pressedElement = null;
				this._touchedElements = {};
				this._touchesForWhichTouchLeaveHasFired = {};
				this._selectedElements = {};
				this._selectedPressedElements = {};
				this._useMouse = !options || options.useMouse !== false;
				this._useTouch = !options || options.useTouch !== false;
				this._useXr = !options || options.useXr !== false;
				this._selectEventsAttached = false;
				if (platform.touch) {
						this._clickedEntities = {};
				}
				this.attach(domElement);
		}
}

export { ElementInput, ElementInputEvent, ElementMouseEvent, ElementSelectEvent, ElementTouchEvent };
