import { LitShader } from './lit-shader.js';
import { LitOptionsUtils } from './lit-options-utils.js';
import { ShaderGenerator } from './shader-generator.js';
import { SHADERLANGUAGE_WGSL, SHADERLANGUAGE_GLSL, SHADERTAG_MATERIAL } from '../../../platform/graphics/constants.js';
import { ShaderUtils } from '../../../platform/graphics/shader-utils.js';
import { hashCode } from '../../../core/hash.js';

function _extends() {
		_extends = Object.assign || function(target) {
				for(var i = 1; i < arguments.length; i++){
						var source = arguments[i];
						for(var key in source){
								if (Object.prototype.hasOwnProperty.call(source, key)) {
										target[key] = source[key];
								}
						}
				}
				return target;
		};
		return _extends.apply(this, arguments);
}
var dummyUvs = [
		0,
		1,
		2,
		3,
		4,
		5,
		6,
		7
];
class ShaderGeneratorLit extends ShaderGenerator {
		generateKey(options) {
				var definesHash = ShaderGenerator.definesHash(options.defines);
				var _options_shaderChunkGLSL;
				var glslHash = hashCode((_options_shaderChunkGLSL = options.shaderChunkGLSL) != null ? _options_shaderChunkGLSL : '');
				var _options_shaderChunkWGSL;
				var wgslHash = hashCode((_options_shaderChunkWGSL = options.shaderChunkWGSL) != null ? _options_shaderChunkWGSL : '');
				var loHash = LitOptionsUtils.generateKey(options.litOptions);
				var uvOptions = "" + dummyUvs.map((dummy, index)=>{
						return options.usedUvs[index] ? '1' : '0';
				}).join('');
				return "lit_" + definesHash + "_" + uvOptions + "_" + glslHash + "_" + wgslHash + "_" + loHash;
		}
		createShaderDefinition(device, options) {
				var wgsl = device.isWebGPU && options.shaderChunkWGSL;
				var shaderLanguage = wgsl ? SHADERLANGUAGE_WGSL : SHADERLANGUAGE_GLSL;
				var litShader = new LitShader(device, options.litOptions, shaderLanguage);
				var definitionOptions = {
						name: 'LitShader',
						shaderLanguage: shaderLanguage,
						tag: litShader.shaderPassInfo.isForward ? SHADERTAG_MATERIAL : undefined
				};
				var usedUvSets = options.usedUvs || [
						true
				];
				var mapTransforms = [];
				litShader.generateVertexShader(usedUvSets, usedUvSets, mapTransforms);
				litShader.generateFragmentShader('', wgsl ? options.shaderChunkWGSL : options.shaderChunkGLSL, 'vUv0');
				var includes = new Map(Object.entries(_extends({}, Object.getPrototypeOf(litShader.chunks), litShader.chunks, options.litOptions.chunks)));
				var vDefines = litShader.vDefines;
				options.defines.forEach((value, key)=>vDefines.set(key, value));
				var fDefines = litShader.fDefines;
				options.defines.forEach((value, key)=>fDefines.set(key, value));
				definitionOptions.attributes = litShader.attributes;
				definitionOptions.vertexCode = litShader.vshader;
				definitionOptions.vertexIncludes = includes;
				definitionOptions.vertexDefines = vDefines;
				definitionOptions.fragmentCode = litShader.fshader;
				definitionOptions.fragmentIncludes = includes;
				definitionOptions.fragmentDefines = fDefines;
				return ShaderUtils.createDefinition(device, definitionOptions);
		}
}
var lit = new ShaderGeneratorLit();

export { lit };
