/**
 * @import { AppBase } from '../app-base.js'
 * @import { GSplatData } from '../../scene/gsplat/gsplat-data.js'
 * @import { GSplatCompressedData } from '../../scene/gsplat/gsplat-compressed-data.js'
 * @import { GSplatSogsData } from '../../scene/gsplat/gsplat-sogs-data.js'
 * @import { SplatMaterialOptions } from '../../scene/gsplat/gsplat-material.js'
 */
/**
 * The resource for the gsplat asset type.
 *
 * @category Graphics
 */
export class GSplatResource {
    /**
     * @param {AppBase} app - The app.
     * @param {GSplatData | GSplatCompressedData | GSplatSogsData} splatData - The splat data.
     * @param {string[]} comments - The PLY file header comments
     * @ignore
     */
    constructor(app: AppBase, splatData: GSplatData | GSplatCompressedData | GSplatSogsData, comments: string[]);
    /**
     * @type {AppBase | null}
     * @ignore
     */
    app: AppBase | null;
    /**
     * @type {GSplatData | GSplatCompressedData | GSplatSogsData }
     * @ignore
     */
    splatData: GSplatData | GSplatCompressedData | GSplatSogsData;
    /**
     * @type {GSplat | GSplatCompressed | GSplatSogs | null}
     * @ignore
     */
    splat: GSplat | GSplatCompressed | GSplatSogs | null;
    /**
     * @type {string[] | null}
     * @ignore
     */
    comments: string[] | null;
    destroy(): void;
    createSplat(): GSplatCompressed | GSplatSogs | GSplat;
    /**
     * Instantiates an entity with a {@link GSplatComponent}.
     *
     * @param {SplatMaterialOptions} [options] - The options.
     * @returns {Entity} The entity with {@link GSplatComponent}.
     */
    instantiate(options?: SplatMaterialOptions): Entity;
    createInstance(options?: {}): GSplatInstance;
}
import type { AppBase } from '../app-base.js';
import type { GSplatData } from '../../scene/gsplat/gsplat-data.js';
import type { GSplatCompressedData } from '../../scene/gsplat/gsplat-compressed-data.js';
import type { GSplatSogsData } from '../../scene/gsplat/gsplat-sogs-data.js';
import { GSplat } from '../../scene/gsplat/gsplat.js';
import { GSplatCompressed } from '../../scene/gsplat/gsplat-compressed.js';
import { GSplatSogs } from '../../scene/gsplat/gsplat-sogs.js';
import type { SplatMaterialOptions } from '../../scene/gsplat/gsplat-material.js';
import { Entity } from '../entity.js';
import { GSplatInstance } from '../../scene/gsplat/gsplat-instance.js';
