import { ShaderUtils } from '../../../platform/graphics/shader-utils.js';
import { blendNames } from '../../constants.js';
import { shaderChunks } from '../chunks/chunks.js';
import { ShaderGenerator } from './shader-generator.js';

function _extends() {
		_extends = Object.assign || function(target) {
				for(var i = 1; i < arguments.length; i++){
						var source = arguments[i];
						for(var key in source){
								if (Object.prototype.hasOwnProperty.call(source, key)) {
										target[key] = source[key];
								}
						}
				}
				return target;
		};
		return _extends.apply(this, arguments);
}
var normalTypeNames = [
		'NONE',
		'VERTEX',
		'MAP'
];
class ShaderGeneratorParticle extends ShaderGenerator {
		generateKey(options) {
				var definesHash = ShaderGenerator.definesHash(options.defines);
				var key = "particle_" + definesHash + "_";
				for(var prop in options){
						if (options.hasOwnProperty(prop)) {
								key += options[prop];
						}
				}
				return key;
		}
		createVertexDefines(options) {
				var vDefines = new Map(options.defines);
				if (options.mesh) vDefines.set('USE_MESH', '');
				if (options.localSpace) vDefines.set('LOCAL_SPACE', '');
				if (options.screenSpace) vDefines.set('SCREEN_SPACE', '');
				if (options.animTex) vDefines.set('ANIMTEX', '');
				if (options.soft > 0) vDefines.set('SOFT', '');
				if (options.stretch > 0.0) vDefines.set('STRETCH', '');
				if (options.customFace) vDefines.set('CUSTOM_FACE', '');
				if (options.pack8) vDefines.set('PACK8', '');
				if (options.localSpace) vDefines.set('LOCAL_SPACE', '');
				if (options.animTexLoop) vDefines.set('ANIMTEX_LOOP', '');
				if (options.wrap) vDefines.set('WRAP', '');
				if (options.alignToMotion) vDefines.set('ALIGN_TO_MOTION', '');
				vDefines.set('NORMAL', normalTypeNames[options.normal]);
				return vDefines;
		}
		createFragmentDefines(options) {
				var fDefines = new Map(options.defines);
				if (options.soft > 0) fDefines.set('SOFT', '');
				if (options.halflambert) fDefines.set('HALF_LAMBERT', '');
				fDefines.set('NORMAL', normalTypeNames[options.normal]);
				fDefines.set('BLEND', blendNames[options.blend]);
				return fDefines;
		}
		createShaderDefinition(device, options) {
				var vDefines = this.createVertexDefines(options);
				var fDefines = this.createFragmentDefines(options);
				var executionDefine = "PARTICLE_" + (options.useCpu ? 'CPU' : 'GPU') + "\n";
				vDefines.set(executionDefine, '');
				fDefines.set(executionDefine, '');
				var includes = new Map(Object.entries(_extends({}, shaderChunks, options.chunks)));
				return ShaderUtils.createDefinition(device, {
						name: 'ParticleShader',
						vertexCode: shaderChunks.particle_shaderVS,
						fragmentCode: shaderChunks.particle_shaderPS,
						fragmentDefines: fDefines,
						fragmentIncludes: includes,
						vertexIncludes: includes,
						vertexDefines: vDefines
				});
		}
}
var particle = new ShaderGeneratorParticle();

export { particle };
