declare const _default: "\n\nattribute vertex_normal: vec3f;\n\n#ifdef MORPHING_NORMAL\n    #ifdef MORPHING_INT\n        uniform morphNormalTex: texture_2d<u32>;\n        uniform morphNormalTexSampler: sampler;\n    #else\n        uniform morphNormalTex: texture_2d<f32>;\n        uniform morphNormalTexSampler: sampler;\n    #endif\n#endif\n\nfn getLocalNormal(vertexNormal: vec3f) -> vec3f {\n\n    var localNormal: vec3f = vertexNormal;\n\n    #ifdef MORPHING_NORMAL\n\n        // Assuming getTextureMorphCoords() returns vec2i\n        let morphUV: vec2i = getTextureMorphCoords();\n\n        #ifdef MORPHING_INT\n            // Use textureLoad (texelFetch equivalent), requires integer coordinates and returns vec4u\n            let morphNormalInt: vec4u = textureLoad(morphNormalTex, morphUV, 0);\n            // Convert unsigned int vector components to float vector for calculations\n            let morphNormalF: vec3f = vec3f(morphNormalInt.xyz) / 65535.0 * 2.0 - 1.0;\n            localNormal = localNormal + morphNormalF; // Expand +=\n        #else\n            // Use textureLoad, requires integer coordinates and returns vec4f\n            let morphNormal: vec3f = textureLoad(morphNormalTex, morphUV, 0).xyz;\n            localNormal = localNormal + morphNormal; // Expand +=\n        #endif\n\n    #endif\n\n    return localNormal;\n}\n\n#ifdef SKIN\n    fn getNormalMatrix(modelMatrix: mat4x4f) -> mat3x3f {\n        // Construct mat3x3f from columns' xyz components\n        return mat3x3f(modelMatrix[0].xyz, modelMatrix[1].xyz, modelMatrix[2].xyz);\n    }\n#elif defined(INSTANCING)\n    fn getNormalMatrix(modelMatrix: mat4x4f) -> mat3x3f {\n        // Construct mat3x3f from columns' xyz components\n        return mat3x3f(modelMatrix[0].xyz, modelMatrix[1].xyz, modelMatrix[2].xyz);\n    }\n#else\n    fn getNormalMatrix(modelMatrix: mat4x4f) -> mat3x3f {\n        // Access uniform via uniform.<name>\n        return uniform.matrix_normal;\n    }\n#endif\n";
export default _default;
