declare const _default: "\nuniform highp sampler2D means_u;\nuniform highp sampler2D means_l;\nuniform highp sampler2D quats;\nuniform highp sampler2D scales;\n\nuniform vec3 means_mins;\nuniform vec3 means_maxs;\n\nuniform vec3 quats_mins;\nuniform vec3 quats_maxs;\n\nuniform vec3 scales_mins;\nuniform vec3 scales_maxs;\n\n// read the model-space center of the gaussian\nvec3 readCenter(SplatSource source) {\n    vec3 u = texelFetch(means_u, source.uv, 0).xyz;\n    vec3 l = texelFetch(means_l, source.uv, 0).xyz;\n    vec3 n = (l * 255.0 + u * 255.0 * 256.0) / 65535.0;\n\n    vec3 v = mix(means_mins, means_maxs, n);\n    return sign(v) * (exp(abs(v)) - 1.0);\n}\n\nmat3 quatToMat3(vec3 R) {\n    float x = R.x;\n    float y = R.y;\n    float z = R.z;\n    float w2 = clamp(1.0 - (x*x + y*y + z*z), 0.0, 1.0);\n    float w  = sqrt(w2);\n    return mat3(\n        1.0 - 2.0 * (z * z + w2),\n              2.0 * (y * z + x * w),\n              2.0 * (y * w - x * z),\n              2.0 * (y * z - x * w),\n        1.0 - 2.0 * (y * y + w2),\n              2.0 * (z * w + x * y),\n              2.0 * (y * w + x * z),\n              2.0 * (z * w - x * y),\n        1.0 - 2.0 * (y * y + z * z)\n    );\n}\n\n// sample covariance vectors\nvoid readCovariance(in SplatSource source, out vec3 covA, out vec3 covB) {\n    vec3 quat = mix(quats_mins, quats_maxs, texelFetch(quats, source.uv, 0).xyz);\n    mat3 rot = quatToMat3(quat);\n    vec3 scale = exp(mix(scales_mins, scales_maxs, texelFetch(scales, source.uv, 0).xyz));\n\n    // M = S * R\n    mat3 M = transpose(mat3(\n        scale.x * rot[0],\n        scale.y * rot[1],\n        scale.z * rot[2]\n    ));\n\n    covA = vec3(dot(M[0], M[0]), dot(M[0], M[1]), dot(M[0], M[2]));\n    covB = vec3(dot(M[1], M[1]), dot(M[1], M[2]), dot(M[2], M[2]));\n}\n";
export default _default;
